## no critic(RCS,VERSION,explicit,Module)
use strict;
use warnings;

use Test::More;


if ( $ENV{RELEASE_TESTING} ) {
    eval 'use Test::Pod::Coverage 1.00';    ## no critic (eval)
    if ($@) {
        plan skip_all =>
          'Test::Pod::Coverage 1.00 required for testing POD coverage';
    }
    diag 'We get a warning generated by Inline about DATA '
      . "sections being processed by Inline.\nIt\'s known to be harmless.";
}
else {
    plan skip_all => 'Skip: Test::Pod::Coverage tests skipped unless '
      . 'environment variable RELEASE_TESTING is set.';
}

pod_coverage_ok( 'Math::Prime::FastSieve' );
done_testing();

